/* $Id$

This file is part of libmspdatafile
Copyright © 2006  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/
#include <cctype>
#include <sstream>
#include "binaryparser.h"
#include "parser.h"
#include "statement.h"
#include "textparser.h"

using namespace std;

namespace Msp {
namespace DataFile {

Parser::Parser(istream &i, const string &s):
	in(i),
	src(s),
	good(true),
	mode(new TextParser(in, src))
{ }

Parser::~Parser()
{
	delete mode;
}

Statement Parser::parse()
{
	if(!good)
		throw Exception("Parser is not good");

	try
	{
		while(1)
		{
			Statement st=mode->parse();
			if(st.keyword=="__bin")
			{
				delete mode;
				mode=new BinaryParser(in, src);
			}
			else if(st.keyword=="__text")
			{
				delete mode;
				mode=new TextParser(in, src);
			}
			else
				return st;
		}
	}
	catch(const Exception &e)
	{
		good=false;
		throw;
	}
}

} // namespace DataFile
} // namespace Msp
